@@ -1,3 +1,4 @@
+from os import fchdir
 from Bio.PDB import *
 from Bio import SeqIO
 import pyvista as pv
@@ -9,7 +10,7 @@ import trimesh
 
 from provis.utils.atminfo import import_atm_size_info, import_atm_mass_info
 from provis.utils.bond_parser import bond_parser
-from provis.utils.name_checker import NameChecker
+from provis.src.processing.name_checker import NameChecker
 from provis.src.processing.file_converter import FileConverter
 
 
@@ -18,16 +19,24 @@ class DataHandler:
     The 'brain' of provis, when it comes to handling atomic positions. 
     
     This class loads information from a variety of files and creates meshes to be plotted. 
-    Upper level classes - eg. StickPoint - have their own AtomHandler objects that do all the work.
+    Upper level classes - eg. Structure - have their own DataHandler objects that do all the work.
     """
-    def __init__(self):
+    def __init__(self, nc, fc=None):
         """
-        Load structure form pdb file (by parsing file) and save Biopython structure representation of the protein. Load all size and color helper dictionaries.
+        Load structure form pdb file (by parsing file) and save Biopython structure representation of the protein. Also loads all size and color helper dictionaries.
         
-        :param name: name - Name of file to be loaded. Passed to name_checker.check_name() to get usable paths.
-        :param type: str
+        :param name: nc - Instance of a NameChecker class. Used to pass the pdb file name and paths.
+        :param type: NameChecker
+        :param name: fc - Instance of a FileConverter class. Needed if temporary files have not been created before instantiating this class. Default: None.
+        :param type: FileConverter, optional
         """
-        self._path, self._out_path, self._base_path = NameChecker.return_all()
+        self._path, self._out_path, self._base_path = nc.return_all()
+        if not fc:
+            self._fc = FileConverter(nc)
+        else:
+            self._fc = fc
+        
+        self._fc.pdb_to_xyzrn(self._path, self._out_path)
         
         parser = PDBParser()
         file_name = self._path + ".pdb"
@@ -48,7 +57,6 @@ class DataHandler:
             "PRO": '#9FEBA4', "SER": '#BBD7EB', "THR": '#D1A67A', "TRP": '#F93713', "TYR": '#E5613D',
             "VAL": '#128033', "HOH": 'w', "HEM": 'r'}
         self._atoms_size_dict, self._atoms_color_dict, self._vw_dict = import_atm_size_info(self._base_path, 1)
-        self.get_backbone_mesh()
 
 
     def get_atoms(self, show_solvent=False):
@@ -58,7 +66,7 @@ class DataHandler:
         :param name: show_solvent - If True solvent molecules also added to retrun dictionary. Default: False.
         :param type: bool, optional
         
-        :return: dict - Dictionary of atomic coordinates by atom type.
+        :returns: dict - Dictionary of atomic coordinates by atom type.
         """
         # load file into a python list
         residues = self._structure.get_residues()
@@ -86,7 +94,9 @@ class DataHandler:
         """
         Get atomic coordinates and residue IDs (format from output_pdb_as_xyzrn()) from the xyzrn file.
 
-        :returns: 
+        :returns: dict - Dictionary of atomic coordinates by atom type.
+        :returns: list - List of unique residue IDs (format from output_pdb_as_xyzrn())
+        :returns: list - Atomic coordinates (in same order as the residue IDs)
         """
         xyzrnfile = open(self._out_path + ".xyzrn")
         meshdata = (xyzrnfile.read().rstrip()).split("\n")
@@ -96,6 +106,7 @@ class DataHandler:
         res_id = [""] * lenm
         
         atom_data = dict()
+        atom_coords = []
         for vi in range(lenm):
             fields = meshdata[vi].split()
             vertices = [None] * 3
@@ -110,16 +121,17 @@ class DataHandler:
             # If atom already in dictionary, append its coordinates to list
             else:
                 atom_data[atmtype].append(vertices)
+            atom_coords.append(vertices)
 
 
         # return the 3D positions of atoms organized by atom type
-        return atom_data, res_id
+        return atom_data, res_id, atom_coords
 
     def get_residues(self):
         """
         Creates a dictionary of coordinates by residues from structure object
         
-        :return: dict - Dictionary of atomic coordinates by residue type.
+        :returns: dict - Dictionary of atomic coordinates by residue type.
         """
 
         # load file into a python list
@@ -154,7 +166,7 @@ class DataHandler:
         :param name: option - choose what property of residue you want. com for Centre Of Mass, ch for charge
         :param type: str - options: com, ch
         
-        :return: list - List of COM coords of given (exact) residue.
+        :returns: list - List of COM coords of given (exact) residue.
         """
         
         # load info for given residue
@@ -203,7 +215,7 @@ class DataHandler:
         """
         Return the loaded structure object
         
-        :return: structure
+        :returns: structure
         """
         return self._structure
         
@@ -223,8 +235,8 @@ class DataHandler:
         :param name: theta_res - pyvista theta_resolution for Sphere objects representing atoms. Default: 10.
         :param type: int, optional
         
-        :return: list - List of pyvista Shperes representing each atom
-        :return: list - List of colors corresponding to each atom
+        :returns: list - List of pyvista Shperes representing each atom
+        :returns: list - List of colors corresponding to each atom
         """
         # these two dictionaries have to be manually created
         # also, if more/other atoms present in protein it will not work
@@ -266,8 +278,8 @@ class DataHandler:
         :param name: probe - size of probe (representing the solvent size) needed for surface calculation. Default: 0.
         :param type: int, optional
         
-        :return: list - List of pyvista Shperes representing each atom
-        :return: list - List of colors for each atom
+        :returns: list - List of pyvista Shperes representing each atom
+        :returns: list - List of colors for each atom
         """
         # these two dictionaries have to be manually created
         # also, if more/other atoms present in protein it will not work
@@ -316,8 +328,8 @@ class DataHandler:
         :param name: theta_res - pyvista theta_resolution for Sphere objects representing atoms. Default:25.
         :param type: int, optional
         
-        :return: list - List of pyvista Shperes representing each residue
-        :return: list - List of colors for each residue
+        :returns: list - List of pyvista Shperes representing each residue
+        :returns: list - List of colors for each residue
         """
 
         # create glyphs (spherical) to represent each res
@@ -359,17 +371,16 @@ class DataHandler:
             Purple for all aromatic bonds,
             Black for everything else.
         
-        :return: list - List of pyvista lines representing each bond.
-        :retrun: list - List of colors corresponding to the lines in the above list
+        :returns: list - List of pyvista lines representing each bond.
+        :retruns: list - List of colors corresponding to the lines in the above list
         """
         fname = self._out_path + ".mol2"
 
-        # # Check if mol2 file exists. If not convert it from pdb
-        # file_exists = exists(fname)
-        # fc = FileConverter()
-        # fc.pdb_to_mol2(self._path, self._out_path)
-        # if not file_exists:
-        #     fc.pdb_to_mol2(self._path, self._out_path)
+        # Check if mol2 file exists. If not convert it from pdb
+        file_exists = exists(fname)
+        if not file_exists:
+            self._fc.pdb_to_mol2(self._path, self._out_path)
+            
         pmol = PandasMol2().read_mol2(fname)
         bonds_in = bond_parser(fname)
 
@@ -426,4 +437,5 @@ class DataHandler:
                     count += 1
                 com /= count
                 res_list.append(com)
+                
         return np.array(res_list)
\ No newline at end of file
