@@ -1,49 +1,41 @@
 import subprocess
 import os.path
-from provis.utils.name_checker import NameChecker
+from provis.src.processing.name_checker import NameChecker
 from provis.utils.surface_utils import output_pdb_as_xyzrn
 
 class FileConverter():
     """
     Class to create and destroy necessary files required in other parts of code.
-    It has a bunch of classmethod functions to call the binaries and scripts to convert the files.
+    It has a bunch of member functions to call the binaries and scripts to convert the files.
     
-    Best practice is to initialize a FileConverter in a given file before calling any function of provis and call the cleanup method after the last plotting function is called. This will keep your directories decluttered. 
+    Also has a cleanup function that removes everything in the "root_directory"/data/tmp (and data/img if specified) directory. Best practice is to call this function at the end of your main file.
     However if you want to plot the same protein many times, then it is benificial to keep the temporary (data/tmp) files as if they exist provis will not recompute them.
     """
     
-    _base_path = ""
-    def __init__(self, name=None, dens=None, solv=0, bash=0, base_path=None):
+    def __init__(self, nc, density=3.0, convert_all=False):
         """
-        Can be constructed empty. If called with arguments conversions instant. Creates xyzr and mol2 files in every case and face and vert files if msms binary exists.
-
-        :param name: name - Name of file to be loaded. Passed to NameChecker() to get usable paths. Default: None.
+        Can be constructed empty. 
+        If "convert_all" set to True conversions instant. Creates xyzr and mol2 files in every case and pqr, face and vert files if appropriate binary exists.
+        
+        :param name: nc - Instance of a NameChecker class. Used to pass the pdb file name and paths.
         :param type: str, optional
+        :param name: density - Density of triangles. Default: 3.0.
         :param type: float, optional
-        :param name: solv - set to True if you want to plot solvent atoms as well. Default: False.
-        :param type: bool, optional
-        :param name: bash - set to True if you want to run binary version of pdb_to_xyzr (comes with msms). Default: False.
-        :param type: bool, optional
         :param name: base_path - Path to "working directory" according to the rules of NameChecker().
-        :param name: dens - Density of triangles. Default: None.
+        :param type: str, optional
+        :param name: convert_all - Set to True if you want to convert all necessairy files on initialization. Default: False.
+        :param type: bool, optional
         """
 
-        self._solv = solv
-        self._bash = bash
-        if name:
-            NameChecker(name, base_path)
-            self._path, self._out_path, FileConverter._base_path = NameChecker.return_all()
+        self._path, self._out_path, self._base_path = nc.return_all()
+        if convert_all:
+            self.pdb_to_xyzrn(self._path, self._out_path)
+            self.msms(self._path, density)
             self.pdb_to_mol2(self._path, self._out_path)
             self.pdb_to_pqr(self._path, self._out_path)
-            self.pdb_to_xyzrn(self._path, self._out_path)
-        if dens:
-            self._dens = dens
-        if name and dens:
-            self.msms(self._out_path, self._dens)
-        pass
+            
     
-    @classmethod
-    def pdb_to_xyzrn(cls, path, output):
+    def pdb_to_xyzrn(self, path, output):
         """
         Converts .pdb to .xyzrn file.
         
@@ -65,8 +57,7 @@ class FileConverter():
         else:
             print("xyzrn files already exist. No conversion needed")
         
-    @classmethod
-    def msms(cls, path, dens):
+    def msms(self, path, dens):
         """
         Run the msms binary for given filename.
         
@@ -90,7 +81,7 @@ class FileConverter():
         
         MSMS_BIN = ""
         if not os.path.exists(MSMS_BIN):
-            MSMS_BIN = FileConverter._base_path + 'binaries/msms'
+            MSMS_BIN = self._base_path + 'binaries/msms'
 
         file_base = f"{path}_out_{int(dens * 10)}"
 
@@ -108,8 +99,7 @@ class FileConverter():
         else:
             print("MSMS Binary not found under: ", MSMS_BIN)
             
-    @classmethod
-    def pdb_to_mol2(cls, path, outpath):
+    def pdb_to_mol2(self, path, outpath):
         """
         Run openbabel, to convert pdb to mol2
         
@@ -125,8 +115,7 @@ class FileConverter():
         else:
             print("mol2 file already exist. No conversion needed")
 
-    @classmethod
-    def pdb_to_pqr(cls, path, outpath, forcefield="swanson"):
+    def pdb_to_pqr(self, path, outpath, forcefield="swanson"):
         """
         Run pdb2pqr, to convert pdb to pqr
         
@@ -147,7 +136,7 @@ class FileConverter():
             
         PDB2PQR_BIN = ""
         if not os.path.exists(PDB2PQR_BIN):
-            PDB2PQR_BIN = FileConverter._base_path + "binaries/pdb2pqr/pdb2pqr"
+            PDB2PQR_BIN = self._base_path + "binaries/pdb2pqr/pdb2pqr"
             
         #possibilities: amber, charmm, parse, tyl06, peoepb and swanson
         args = [PDB2PQR_BIN , f"--ff={forcefield}", f"{path}.pdb", f"{outpath}.pqr"]
@@ -164,8 +153,7 @@ class FileConverter():
             print("PDB2PQR Binary not found under: ", PDB2PQR_BIN)
        
 
-    @classmethod
-    def cleanup(cls, delete_img: bool=False):
+    def cleanup(self, delete_img: bool=False):
         """
         Deletes all files from data/tmp (and data/img) directories.
 
@@ -174,10 +162,10 @@ class FileConverter():
         """
 
         import os, shutil
-        tmp = FileConverter._base_path + "data/tmp/"
+        tmp = self._base_path + "data/tmp/"
         folders = [tmp]
         if delete_img:
-            img = FileConverter._base_path + "data/img/"
+            img = self._base_path + "data/img/"
             folders.append(img)
         for folder in folders:
             for filename in os.listdir(folder):
