@@ -11,11 +11,13 @@ from pyvtk import PolyData, PointData, CellData, Scalars, Vectors, VtkData
 import trimesh
 from subprocess import PIPE, Popen
 import open3d as o3d
+import enum
+from os.path import exists
 
-from provis.utils.surface_utils import get_surface, compute_normal, prepare_trimesh, fix_trimesh
-from provis.utils.name_checker import NameChecker
-from provis.src.processing.data_handler import DataHandler
+from provis.utils.surface_utils import get_surface, compute_normal, prepare_trimesh, fix_trimesh, find_nearest_atom
 from provis.utils.surface_feat import compute_surface_features
+from provis.src.processing.file_converter import FileConverter
+from provis.src.processing.data_handler import DataHandler
 
 class SurfaceHandler:
     """
@@ -25,32 +27,51 @@ class SurfaceHandler:
     Upper level classes - eg. StickPoint - have their own AtomHandler objects that do all the work.    
     """
   
-    def __init__(self, dens=None):
+    def __init__(self, nc, fc=None, dh=None, density=3.0):
         """
         Initializes SurfaceHandler class memeber variables: 
-        _dh: DataHandler - needed for native mesh creation
-        _path: str - path to pdb file (without extension)
-        _out_path: str - path to output files (without extension)
-        _density: float - density of surface mesh
+        _dh: DataHandler - needed for native mesh creation.
+        _fc: FileConverter - needed to create necessairy files.   
+        _path: str - path to pdb file (without extension).
+        _out_path: str - path to output files (without extension).
+        _base_path: str - path to root directory.
+        _density: float - density of surface mesh.
         _features: tuple(numpy.ndarray) - A collection of arrays representing the feature information for the surface of the protein, saved in a tuple.
-        _mesh: Mesh - Surface mesh of protein
+        _mesh: Mesh - Surface mesh of protein.
         _col: numpy.ndarray - Coloring map of surface. To be passed to pyvista.PolyData.add_mesh(scalars=).
+        _res_id: list - List of residue IDs (format from output_pdb_as_xyzrn())
+        _atom_coords: list - List of atomic coordinates. Just the coordinates in the same order as in _res_id.
 
-        :param name: dens - Density needed for msms. Defaults to None.
+        :param name: nc - Instance of a NameChecker class. Used to pass the pdb file name and paths.
+        :param type: NameChecker
+        :param name: fc - Instance of a FileConverter class. Used to create the necessairy files (face, vert, pqr, ...) if they do not exist yet. Default: None.
+        :param type: FileConverter
+        :param name: dh - Instance of a DataHandler class. Used to retrieve atom-positional information when calculating the surface of the protein natively. Default: None. If None a new DataHandler variable will be initialized with "nc".
+        :param type: DataHandler, optional
+        :param name: density - Density needed for msms. Default: 3.0.
         :param type: float, optional
         """
-        self._dh = DataHandler()
-        self._path, self._out_path, self._base_path = NameChecker.return_all()
-        if dens:
-            self._density = dens
+        
+        if not fc:
+            self._fc = FileConverter(nc)
+        else:
+            self._fc = fc
+        if not dh:
+            self._dh = DataHandler(nc, fc)
+        else:
+            self._dh = dh
+        self._path, self._out_path, self._base_path = nc.return_all()
+        self._density = density
         self._features = None
         self._mesh = None
         self._col = None
+        self._res_id = None
+        self._atom_coords = None
         
     def get_assignments(self):
         """
         Get assignments (coloring) for the mesh. File has to exist, no way to produce it with provis. 
-        Loads provis/data/tmp/{pdb_id}.pth and returns it.
+        Loads "root directory"/data/tmp/{pdb_id}.pth and returns it.
 
         :returns: PyTourch object - Coloring of surface.
         """
@@ -59,7 +80,7 @@ class SurfaceHandler:
         
         return assigment
 
-    def get_surface_features(self, mesh, feature):
+    def get_surface_features(self, mesh, feature,  res_id=None):
         """
         Get the coloring corresponding to a specific feature.
 
@@ -67,6 +88,8 @@ class SurfaceHandler:
         :param type: Trimesh
         :param name: feature - Name of feature we are interested in. Options: hydrophob, shape, charge.
         :param type: str
+        :param name: native - Set to True when using native mesh computation. Default: False.
+        :param type: bool, optional
         
         :raises: NotImplementedError - If unkown feature specified error is raised
 
@@ -75,12 +98,20 @@ class SurfaceHandler:
 
         # get surface
         pdb_file = self._path + '.pdb'
-        surface = get_surface(self._out_path, density=self._density)
-
+        
+        pqr_file = self._out_path + '.pqr'
+        if not exists(pqr_file):
+            self._fc.pdb_to_pqr(self._path, self._out_path)
+        
+        if not res_id:
+            surface = get_surface(self._out_path, density=self._density)
+        else:
+            full_res_id = find_nearest_atom(self._atom_coords, res_id, mesh.vertices)
+            
+            surface = [mesh.vertices, mesh.faces, mesh.face_normals, full_res_id, mesh.area_faces]
+            
         # compute features of surface
         if not self._features:
-            # print(surface)
-            # print(mesh)
             self._features = compute_surface_features(surface, pdb_file, self._out_path, mesh, pdb_id = self._path)
 
         if feature == 'hydrophob':
@@ -105,6 +136,15 @@ class SurfaceHandler:
         :returns: numpy.ndarray - Color map corresponding to specified feature. Only returned if a feature is specified as an input. This map can be passed to a pyvista.Plotter.add_mesh() function as the 'scalars' argument to get the encoded coloring.
         """
         
+        path = f"{self._out_path}_out_{int(self._density * 10)}"
+        face = path + '.face'
+        vert = path + '.vert'
+        from os.path import exists
+        # Check if face file exists. If not convert it from pdb
+        file_exists = exists(face) and exists(vert)
+        if not file_exists:
+            self._fc.msms(self._out_path, self._density)
+        
         if not self._mesh:
             surface = get_surface(out_path=self._out_path, density=self._density)
             self._mesh = prepare_trimesh(vertices=surface[0], faces=surface[1], normals=surface[2], 
@@ -130,24 +170,24 @@ class SurfaceHandler:
         :returns: numpy.ndarray - Color map corresponding to specified feature. Only returned if a feature is specified as an input. This map can be passed to a pyvista.Plotter.add_mesh() function as the 'scalars' argument to get the encoded coloring.
         """
         if not self._mesh:
-            #old      
-            atom_data = self._dh.get_atoms()
+            # create rough surface by combining vw radii of each atom
+            atom_data, self._res_id, self._atom_coords = self._dh.get_atoms_IDs()
             self._atmsurf, col = self._dh.get_atom_mesh(atom_data, vw=1, probe=0.1)
-            #old
 
-            # WORKING
             # adding the spheres (by atom type) one at a time
             j = 0
             mesh_ = pv.wrap(self._atmsurf[0])
             for mesh in self._atmsurf[1:]:
                 mesh_ = mesh_ + (mesh)
 
+            # blur the spheres and extract edges (pyvista)
             mesh_ = mesh_.delaunay_3d(alpha=1.5).extract_feature_edges()
-            # self._atmsurf, col = self._dh.get_atom_trimesh(atom_data, vw=1, probe=0.1)
-            new = trimesh.Trimesh(mesh_.points)#trimesh.util.concatenate(mesh_)
+            # create trimesh mesh
+            new = trimesh.Trimesh(mesh_.points)
             cloud = o3d.geometry.PointCloud()
             cloud.points = o3d.utility.Vector3dVector(new.vertices)
             cloud.normals = o3d.utility.Vector3dVector(new.vertex_normals)
+            # reconstruct the surface mesh as a trimesh mesh
             radii = [0.1, 0.3, 0.45, 0.6, 0.75, 0.9,1.2, 1.5]
             tri_mesh= o3d.geometry.TriangleMesh.create_from_point_cloud_ball_pivoting(cloud, o3d.utility.DoubleVector(radii))#, depth=depth, width=width, scale=scale, linear_fit=linear_fit)
             v = np.asarray(tri_mesh.vertices)
@@ -155,9 +195,8 @@ class SurfaceHandler:
             f = np.c_[np.full(len(f), 3), f]
             mesh = pv.PolyData(v, f)
             shell =  mesh.clean().reconstruct_surface()#.clean()
-            # WORKING
             shell.compute_normals(inplace=True)
-            
+
             # parse list of PolyData (format) faces and convert them into Trimesh (format) faces 
             len_f = len(shell.faces)
             faces_ = []
@@ -179,7 +218,7 @@ class SurfaceHandler:
 
         # only needed if feature is specified (-> color map needs to be calculated)
         if feature:
-            self._col = self.get_surface_features(self._mesh, feature)
+            self._col = self.get_surface_features(self._mesh, feature, self._res_id)
         else:
             self._col = None
 
